package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

package .tmp;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function3;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.attributeslib.AttributesLib;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.UUID;
import java.util.function.BiConsumer;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.world.entity.ai.attributes.Attribute;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.item.ItemStack;

public class MultiAttrBonus extends GemBonus {
  public static Codec<dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.MultiAttrBonus> CODEC;
  
  protected final List<ModifierInst> modifiers;
  
  protected final String desc;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)ModifierInst.CODEC.listOf().fieldOf("modifiers").forGetter(()), (App)Codec.STRING.fieldOf("desc").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.MultiAttrBonus::new));
  }
  
  public MultiAttrBonus(GemClass gemClass, List<ModifierInst> modifiers, String desc) {
    super(Apotheosis.loc("multi_attribute"), gemClass);
    this.modifiers = modifiers;
    this.desc = desc;
  }
  
  public void addModifiers(ItemStack gem, LootRarity rarity, BiConsumer<Attribute, AttributeModifier> map) {
    List<UUID> uuids = GemItem.getUUIDs(gem);
    int i = 0;
    for (ModifierInst modifier : this.modifiers)
      map.accept(modifier.attr, modifier.build(uuids.get(i++), rarity)); 
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    Object[] values = new Object[this.modifiers.size() * 2];
    int i = 0;
    for (ModifierInst modifier : this.modifiers) {
      values[i] = IFormattableAttribute.toComponent(modifier.attr, modifier.build(UUID.randomUUID(), rarity), AttributesLib.getTooltipFlag());
      values[this.modifiers.size() + i] = IFormattableAttribute.toValueComponent(modifier.attr, modifier.op, i, AttributesLib.getTooltipFlag());
      i++;
    } 
    return (Component)Component.m_237110_(this.desc, values).m_130940_(ChatFormatting.YELLOW);
  }
  
  public dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.MultiAttrBonus validate() {
    Preconditions.checkNotNull(this.modifiers, "Invalid AttributeBonus with null values");
    List<Set<LootRarity>> rarityChecks = new ArrayList<>();
    for (ModifierInst inst : this.modifiers) {
      HashSet<LootRarity> set = new HashSet<>();
      Objects.requireNonNull(set);
      RarityRegistry.INSTANCE.getValues().stream().filter(r -> inst.values.containsKey(r)).forEach(set::add);
      rarityChecks.add(set);
    } 
    Preconditions.checkArgument(rarityChecks.stream().mapToInt(Set::size).allMatch(size -> (size == ((Set)rarityChecks.get(0)).size())));
    return this;
  }
  
  public boolean supports(LootRarity rarity) {
    return ((ModifierInst)this.modifiers.get(0)).values.containsKey(rarity);
  }
  
  public int getNumberOfUUIDs() {
    return this.modifiers.size();
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
}
